@@ -246,7 +246,7 @@ gdk_input_wintab_init (void)
       wa.window_type = GDK_WINDOW_TOPLEVEL;
       if ((wintab_window = gdk_window_new (NULL, &wa, GDK_WA_X|GDK_WA_Y)) == NULL)
 	{
-	  g_warning ("_gdk_input_init: gdk_window_new failed");
+	  g_warning ("gdk_input_wintab_init: gdk_window_new failed");
 	  return;
 	}
       gdk_drawable_ref (wintab_window);
@@ -330,7 +330,7 @@ gdk_input_wintab_init (void)
 	  hctx = g_new (HCTX, 1);
           if ((*hctx = WTOpen (GDK_WINDOW_HWND (wintab_window), &lc, TRUE)) == NULL)
 	    {
-	      g_warning ("_gdk_input_init: WTOpen failed");
+	      g_warning ("gdk_input_wintab_init: WTOpen failed");
 	      return;
 	    }
 	  GDK_NOTE (INPUT, g_print ("opened Wintab device %d %p\n",
@@ -566,8 +566,8 @@ gdk_input_translate_coordinates (GdkDevicePrivate *gdkdev,
 
   if (gdkdev->info.mode == GDK_MODE_SCREEN) 
     {
-      x_scale = gdk_screen_width() / device_width;
-      y_scale = gdk_screen_height() / device_height;
+      x_scale = GetSystemMetrics (SM_CXSCREEN) / device_width;
+      y_scale = GetSystemMetrics (SM_CYSCREEN) / device_height;
 
       x_offset = - input_window->root_x;
       y_offset = - input_window->root_y;
@@ -846,7 +846,7 @@ _gdk_input_other_event (GdkEvent  *event,
       if (event->any.type == GDK_BUTTON_PRESS
 	  || event->any.type == GDK_BUTTON_RELEASE)
 	{
-	  event->button.time = msg->time;
+	  event->button.time = _gdk_win32_get_next_tick (msg->time);
 	  event->button.device = &gdkdev->info;
 	  
 #if 0
@@ -875,7 +875,7 @@ _gdk_input_other_event (GdkEvent  *event,
 	}
       else
 	{
-	  event->motion.time = msg->time;
+	  event->motion.time = _gdk_win32_get_next_tick (msg->time);
 	  event->motion.is_hint = FALSE;
 	  event->motion.device = &gdkdev->info;
 
@@ -944,7 +944,7 @@ _gdk_input_other_event (GdkEvent  *event,
 	  event->proximity.type = GDK_PROXIMITY_IN;
 	  _gdk_input_ignore_core = TRUE;
 	}
-      event->proximity.time = msg->time;
+      event->proximity.time = _gdk_win32_get_next_tick (msg->time);
       event->proximity.device = &gdkdev->info;
 
       GDK_NOTE (EVENTS_OR_INPUT,
@@ -1000,10 +1000,10 @@ _gdk_input_grab_pointer (GdkWindow    *window,
   new_window = NULL;
   need_ungrab = FALSE;
 
-  GDK_NOTE (INPUT, g_print ("gdk_input_win32_grab_pointer: %#x %d %#x\n",
-			   (guint) GDK_WINDOW_HWND (window),
+  GDK_NOTE (INPUT, g_print ("gdk_input_win32_grab_pointer: %p %d %p\n",
+			   GDK_WINDOW_HWND (window),
 			   owner_events,
-			   (confine_to ? (guint) GDK_WINDOW_HWND (confine_to) : 0)));
+			   (confine_to ? GDK_WINDOW_HWND (confine_to) : 0)));
 
   while (tmp_list)
     {
