@@ -46,7 +46,6 @@ struct _GtkLabelSelectionInfo
   GdkWindow *window;
   gint selection_anchor;
   gint selection_end;
-  GdkGC *cursor_gc;
   GtkWidget *popup_menu;
 };
 
@@ -540,8 +539,8 @@ gtk_label_get_property (GObject     *object,
     case PROP_CURSOR_POSITION:
       if (label->select_info)
 	{
-	  gint offset = g_utf8_pointer_to_offset (label->label,
-						  label->label + label->select_info->selection_end);
+	  gint offset = g_utf8_pointer_to_offset (label->text,
+						  label->text + label->select_info->selection_end);
 	  g_value_set_int (value, offset);
 	}
       else
@@ -550,8 +549,8 @@ gtk_label_get_property (GObject     *object,
     case PROP_SELECTION_BOUND:
       if (label->select_info)
 	{
-	  gint offset = g_utf8_pointer_to_offset (label->label,
-						  label->label + label->select_info->selection_anchor);
+	  gint offset = g_utf8_pointer_to_offset (label->text,
+						  label->text + label->select_info->selection_anchor);
 	  g_value_set_int (value, offset);
 	}
       else
@@ -909,11 +908,15 @@ gtk_label_set_text (GtkLabel    *label,
 {
   g_return_if_fail (GTK_IS_LABEL (label));
   
+  g_object_freeze_notify (G_OBJECT (label));
+
   gtk_label_set_label_internal (label, g_strdup (str ? str : ""));
   gtk_label_set_use_markup_internal (label, FALSE);
   gtk_label_set_use_underline_internal (label, FALSE);
   
   gtk_label_recalculate (label);
+
+  g_object_thaw_notify (G_OBJECT (label));
 }
 
 /**
@@ -1324,6 +1327,51 @@ gtk_label_clear_layout (GtkLabel *label)
     }
 }
 
+typedef struct _LabelWrapWidth LabelWrapWidth;
+struct _LabelWrapWidth
+{
+  gint width;
+  PangoFontDescription *font_desc;
+};
+
+static void
+label_wrap_width_free (gpointer data)
+{
+  LabelWrapWidth *wrap_width = data;
+  pango_font_description_free (wrap_width->font_desc);
+  g_free (wrap_width);
+}
+
+static gint
+get_label_wrap_width (GtkLabel *label)
+{
+  PangoLayout *layout;
+  GtkStyle *style = GTK_WIDGET (label)->style;
+
+  LabelWrapWidth *wrap_width = g_object_get_data (G_OBJECT (style), "gtk-label-wrap-width");
+  if (!wrap_width)
+    {
+      wrap_width = g_new0 (LabelWrapWidth, 1);
+      g_object_set_data_full (G_OBJECT (style), "gtk-label-wrap-width",
+			      wrap_width, label_wrap_width_free);
+    }
+
+  if (wrap_width->font_desc && pango_font_description_equal (wrap_width->font_desc, style->font_desc))
+    return wrap_width->width;
+
+  if (wrap_width->font_desc)
+    pango_font_description_free (wrap_width->font_desc);
+
+  wrap_width->font_desc = pango_font_description_copy (style->font_desc);
+
+  layout = gtk_widget_create_pango_layout (GTK_WIDGET (label), 
+					   "This long string gives a good enough length for any line to have.");
+  pango_layout_get_size (layout, &wrap_width->width, NULL);
+  g_object_unref (layout);
+
+  return wrap_width->width;
+}
+
 static void
 gtk_label_ensure_layout (GtkLabel *label)
 {
@@ -1378,6 +1426,8 @@ gtk_label_ensure_layout (GtkLabel *label)
 	    pango_layout_set_width (label->layout, aux_info->width * PANGO_SCALE);
 	  else
 	    {
+	      gint wrap_width;
+	      
 	      pango_layout_set_width (label->layout, -1);
 	      pango_layout_get_extents (label->layout, NULL, &logical_rect);
 
@@ -1385,15 +1435,15 @@ gtk_label_ensure_layout (GtkLabel *label)
 	      
 	      /* Try to guess a reasonable maximum width */
 	      longest_paragraph = width;
-	      
-	      width = MIN (width,
-			   PANGO_SCALE * gdk_string_width (gtk_style_get_font (GTK_WIDGET (label)->style),
-							   "This long string gives a good enough length for any line to have."));
+
+	      wrap_width = get_label_wrap_width (label);
+	      width = MIN (width, wrap_width);
 	      width = MIN (width,
 			   PANGO_SCALE * (gdk_screen_width () + 1) / 2);
 	      
 	      pango_layout_set_width (label->layout, width);
 	      pango_layout_get_extents (label->layout, NULL, &logical_rect);
+	      width = logical_rect.width;
 	      height = logical_rect.height;
 	      
 	      /* Unfortunately, the above may leave us with a very unbalanced looking paragraph,
@@ -1412,7 +1462,7 @@ gtk_label_ensure_layout (GtkLabel *label)
 		      pango_layout_get_extents (label->layout, NULL, &logical_rect);
 		      
 		      if (logical_rect.height <= height)
-			width = perfect_width;
+			width = logical_rect.width;
 		      else
 			{
 			  gint mid_width = (perfect_width + width) / 2;
@@ -1423,7 +1473,7 @@ gtk_label_ensure_layout (GtkLabel *label)
 			      pango_layout_get_extents (label->layout, NULL, &logical_rect);
 			      
 			      if (logical_rect.height <= height)
-				width = mid_width;
+				width = logical_rect.width;
 			    }
 			}
 		    }
@@ -1630,8 +1680,7 @@ gtk_label_draw_cursor (GtkLabel  *label, gint xoffset, gint yoffset)
       GdkRectangle cursor_location;
       GtkTextDirection dir1 = GTK_TEXT_DIR_NONE;
       GtkTextDirection dir2 = GTK_TEXT_DIR_NONE;
-      GdkGC *gc1 = NULL;
-      GdkGC *gc2 = NULL;
+      GdkGC *gc;
 
       keymap_direction =
 	(gdk_keymap_get_direction (gdk_keymap_get_default ()) == PANGO_DIRECTION_LTR) ?
@@ -1648,25 +1697,21 @@ gtk_label_draw_cursor (GtkLabel  *label, gint xoffset, gint yoffset)
 		    "gtk-split-cursor", &split_cursor,
 		    NULL);
 
+      dir1 = widget_direction;
+      
       if (split_cursor)
 	{
-	  gc1 = label->select_info->cursor_gc;
 	  cursor1 = &strong_pos;
 
 	  if (strong_pos.x != weak_pos.x ||
 	      strong_pos.y != weak_pos.y)
 	    {
-	      dir1 = widget_direction;
 	      dir2 = (widget_direction == GTK_TEXT_DIR_LTR) ? GTK_TEXT_DIR_RTL : GTK_TEXT_DIR_LTR;
-	      
-	      gc2 = widget->style->black_gc;
 	      cursor2 = &weak_pos;
 	    }
 	}
       else
 	{
-	  gc1 = label->select_info->cursor_gc;
-	  
 	  if (keymap_direction == widget_direction)
 	    cursor1 = &strong_pos;
 	  else
@@ -1677,19 +1722,24 @@ gtk_label_draw_cursor (GtkLabel  *label, gint xoffset, gint yoffset)
       cursor_location.y = yoffset + PANGO_PIXELS (cursor1->y);
       cursor_location.width = 0;
       cursor_location.height = PANGO_PIXELS (cursor1->height);
+
+      gc = _gtk_get_insertion_cursor_gc (widget, TRUE);
+      _gtk_draw_insertion_cursor (widget, widget->window, gc,
+				  &cursor_location, dir1,
+                                  dir2 != GTK_TEXT_DIR_NONE);
+      g_object_unref (gc);
       
-      _gtk_draw_insertion_cursor (widget, widget->window, gc1,
-				  &cursor_location, dir1);
-      
-      if (gc2)
+      if (dir2 != GTK_TEXT_DIR_NONE)
 	{
 	  cursor_location.x = xoffset + PANGO_PIXELS (cursor2->x);
 	  cursor_location.y = yoffset + PANGO_PIXELS (cursor2->y);
 	  cursor_location.width = 0;
 	  cursor_location.height = PANGO_PIXELS (cursor2->height);
-	  
-	  _gtk_draw_insertion_cursor (widget, widget->window, gc2,
-				      &cursor_location, dir2);
+
+	  gc = _gtk_get_insertion_cursor_gc (widget, FALSE);
+	  _gtk_draw_insertion_cursor (widget, widget->window, gc,
+				      &cursor_location, dir2, TRUE);
+	  g_object_unref (gc);
 	}
     }
 }
@@ -1790,8 +1840,10 @@ gtk_label_set_uline_text_internal (GtkLabel    *label,
   g_return_if_fail (GTK_IS_LABEL (label));
   g_return_if_fail (str != NULL);
 
-  /* Convert text to wide characters */
-
+  /* Split text into the base text and a separate pattern
+   * of underscores.
+   */
+  
   new_str = g_new (gchar, strlen (str) + 1);
   pattern = g_new (gchar, g_utf8_strlen (str, -1) + 1);
   
@@ -1873,6 +1925,8 @@ gtk_label_parse_uline (GtkLabel    *label,
   g_return_val_if_fail (str != NULL, GDK_VoidSymbol);
 
   orig_keyval = label->mnemonic_keyval;
+
+  g_object_freeze_notify (G_OBJECT (label));
   
   gtk_label_set_label_internal (label, g_strdup (str ? str : ""));
   gtk_label_set_use_markup_internal (label, FALSE);
@@ -1885,6 +1939,8 @@ gtk_label_parse_uline (GtkLabel    *label,
   
   gtk_label_setup_mnemonic (label, orig_keyval);
   
+  g_object_thaw_notify (G_OBJECT (label));
+
   return keyval;
 }
 
@@ -1909,7 +1965,9 @@ gtk_label_set_text_with_mnemonic (GtkLabel    *label,
   g_return_if_fail (str != NULL);
 
   last_keyval = label->mnemonic_keyval;
-  
+
+  g_object_freeze_notify (G_OBJECT (label));
+
   gtk_label_set_label_internal (label, g_strdup (str ? str : ""));
   gtk_label_set_use_markup_internal (label, FALSE);
   gtk_label_set_use_underline_internal (label, TRUE);
@@ -1917,26 +1975,8 @@ gtk_label_set_text_with_mnemonic (GtkLabel    *label,
   gtk_label_recalculate (label);
 
   gtk_label_setup_mnemonic (label, last_keyval);
-}
 
-static void
-gtk_label_realize_cursor_gc (GtkLabel *label)
-{
-  GdkColor *cursor_color;
-  GdkColor red = {0, 0xffff, 0x0000, 0x0000};
-
-  if (label->select_info == NULL)
-    return;
-  
-  if (label->select_info->cursor_gc)
-    gdk_gc_unref (label->select_info->cursor_gc);
-
-  gtk_widget_style_get (GTK_WIDGET (label), "cursor-color", &cursor_color, NULL);
-  label->select_info->cursor_gc = gdk_gc_new (GTK_WIDGET (label)->window);
-  if (cursor_color)
-    gdk_gc_set_rgb_fg_color (label->select_info->cursor_gc, cursor_color);
-  else
-    gdk_gc_set_rgb_fg_color (label->select_info->cursor_gc, &red);
+  g_object_thaw_notify (G_OBJECT (label));
 }
 
 static void
@@ -1949,23 +1989,7 @@ gtk_label_realize (GtkWidget *widget)
   (* GTK_WIDGET_CLASS (parent_class)->realize) (widget);
 
   if (label->select_info)
-    {
-      gtk_label_create_window (label);
-      gtk_label_realize_cursor_gc (label);
-    }
-}
-
-static void
-gtk_label_unrealize_cursor_gc (GtkLabel *label)
-{
-  if (label->select_info == NULL)
-    return;
-  
-  if (label->select_info->cursor_gc)
-    {
-      gdk_gc_unref (label->select_info->cursor_gc);
-      label->select_info->cursor_gc = NULL;
-    }
+    gtk_label_create_window (label);
 }
 
 static void
@@ -1976,10 +2000,7 @@ gtk_label_unrealize (GtkWidget *widget)
   label = GTK_LABEL (widget);
 
   if (label->select_info)
-    {
-      gtk_label_unrealize_cursor_gc (label);
-      gtk_label_destroy_window (label);
-    }
+    gtk_label_destroy_window (label);
   
   (* GTK_WIDGET_CLASS (parent_class)->unrealize) (widget);
 }
@@ -2137,7 +2158,7 @@ gtk_label_button_press (GtkWidget      *widget,
 
       if (event->type == GDK_3BUTTON_PRESS)
 	{
-	  gtk_label_select_region_index (label, 0, strlen (label->label));
+	  gtk_label_select_region_index (label, 0, strlen (label->text));
 	  return TRUE;
 	}
       
@@ -2177,7 +2198,7 @@ gtk_label_button_press (GtkWidget      *widget,
       else
 	{
 	  if (event->type == GDK_3BUTTON_PRESS)
-	      gtk_label_select_region_index (label, 0, strlen (label->label));
+	      gtk_label_select_region_index (label, 0, strlen (label->text));
 	  else if (event->type == GDK_2BUTTON_PRESS)
 	      gtk_label_select_word (label);
 	  else 
@@ -2326,10 +2347,7 @@ gtk_label_set_selectable (GtkLabel *label,
 	  GTK_WIDGET_SET_FLAGS (label, GTK_CAN_FOCUS);
       
           if (GTK_WIDGET_REALIZED (label))
-	    {
-	      gtk_label_create_window (label);
-	      gtk_label_realize_cursor_gc (label);
-	    }
+	    gtk_label_create_window (label);
 
           if (GTK_WIDGET_MAPPED (label))
             gdk_window_show (label->select_info->window);
@@ -2342,8 +2360,6 @@ gtk_label_set_selectable (GtkLabel *label,
           /* unselect, to give up the selection */
           gtk_label_select_region (label, 0, 0);
           
-	  gtk_label_unrealize_cursor_gc (label);
-	  
           if (label->select_info->window)
 	    {
 	      gtk_label_destroy_window (label);
@@ -2771,10 +2787,10 @@ gtk_label_move_logically (GtkLabel *label,
 			  gint      start,
 			  gint      count)
 {
-  gint offset = g_utf8_pointer_to_offset (label->label,
-					  label->label + start);
+  gint offset = g_utf8_pointer_to_offset (label->text,
+					  label->text + start);
 
-  if (label->label)
+  if (label->text)
     {
       PangoLogAttr *log_attrs;
       gint n_attrs;
@@ -2782,7 +2798,7 @@ gtk_label_move_logically (GtkLabel *label,
 
       gtk_label_ensure_layout (label);
       
-      length = g_utf8_strlen (label->label, -1);
+      length = g_utf8_strlen (label->text, -1);
 
       pango_layout_get_log_attrs (label->layout, &log_attrs, &n_attrs);
 
@@ -2806,7 +2822,7 @@ gtk_label_move_logically (GtkLabel *label,
       g_free (log_attrs);
     }
 
-  return g_utf8_offset_to_pointer (label->label, offset) - label->label;
+  return g_utf8_offset_to_pointer (label->text, offset) - label->text;
 }
 
 static gint
@@ -2858,7 +2874,7 @@ gtk_label_move_visually (GtkLabel *label,
       index = new_index;
       
       while (new_trailing--)
-	index = g_utf8_next_char (label->label + new_index) - label->label;
+	index = g_utf8_next_char (label->text + new_index) - label->text;
     }
   
   return index;
@@ -2868,11 +2884,11 @@ static gint
 gtk_label_move_forward_word (GtkLabel *label,
 			     gint      start)
 {
-  gint new_pos = g_utf8_pointer_to_offset (label->label,
-					   label->label + start);
+  gint new_pos = g_utf8_pointer_to_offset (label->text,
+					   label->text + start);
   gint length;
 
-  length = g_utf8_strlen (label->label, -1);
+  length = g_utf8_strlen (label->text, -1);
   if (new_pos < length)
     {
       PangoLogAttr *log_attrs;
@@ -2890,7 +2906,7 @@ gtk_label_move_forward_word (GtkLabel *label,
       g_free (log_attrs);
     }
 
-  return g_utf8_offset_to_pointer (label->label, new_pos) - label->label;
+  return g_utf8_offset_to_pointer (label->text, new_pos) - label->text;
 }
 
 
@@ -2898,11 +2914,11 @@ static gint
 gtk_label_move_backward_word (GtkLabel *label,
 			      gint      start)
 {
-  gint new_pos = g_utf8_pointer_to_offset (label->label,
-					   label->label + start);
+  gint new_pos = g_utf8_pointer_to_offset (label->text,
+					   label->text + start);
   gint length;
 
-  length = g_utf8_strlen (label->label, -1);
+  length = g_utf8_strlen (label->text, -1);
   
   if (new_pos > 0)
     {
@@ -2922,7 +2938,7 @@ gtk_label_move_backward_word (GtkLabel *label,
       g_free (log_attrs);
     }
 
-  return g_utf8_offset_to_pointer (label->label, new_pos) - label->label;
+  return g_utf8_offset_to_pointer (label->text, new_pos) - label->text;
 }
 
 static void
@@ -2975,7 +2991,7 @@ gtk_label_move_cursor (GtkLabel       *label,
 	case GTK_MOVEMENT_PARAGRAPH_ENDS:
 	case GTK_MOVEMENT_BUFFER_ENDS:
 	  /* FIXME: Can do better here */
-	  new_pos = count < 0 ? 0 : strlen (label->label);
+	  new_pos = count < 0 ? 0 : strlen (label->text);
 	  break;
 	case GTK_MOVEMENT_DISPLAY_LINES:
 	case GTK_MOVEMENT_PARAGRAPHS:
@@ -3009,7 +3025,7 @@ gtk_label_move_cursor (GtkLabel       *label,
 	case GTK_MOVEMENT_PARAGRAPH_ENDS:
 	case GTK_MOVEMENT_BUFFER_ENDS:
 	  /* FIXME: Can do better here */
-	  new_pos = count < 0 ? 0 : strlen (label->label);
+	  new_pos = count < 0 ? 0 : strlen (label->text);
 	  break;
 	case GTK_MOVEMENT_DISPLAY_LINES:
 	case GTK_MOVEMENT_PARAGRAPHS:
@@ -3056,7 +3072,7 @@ gtk_label_copy_clipboard (GtkLabel *label)
 static void
 gtk_label_select_all (GtkLabel *label)
 {
-  gtk_label_select_region_index (label, 0, strlen (label->label));
+  gtk_label_select_region_index (label, 0, strlen (label->text));
 }
 
 /* Quick hack of a popup menu
