@@ -101,8 +101,8 @@ struct _GtkTreeViewPrivate
   GdkWindow *drag_highlight_window;
   GtkTreeViewColumn *drag_column;
 
-  GtkTreeRowReference *last_single_clicked;
-  GtkTreeRowReference *last_single_clicked_2;
+  GtkTreeRowReference *last_button_press;
+  GtkTreeRowReference *last_button_press_2;
 
   /* bin_window offset */
   GtkTreeRowReference *top_row;
@@ -272,7 +272,8 @@ void         _gtk_tree_selection_internal_select_node (GtkTreeSelection  *select
 						       GtkRBNode         *node,
 						       GtkRBTree         *tree,
 						       GtkTreePath       *path,
-						       GdkModifierType    state);
+						       GdkModifierType    state,
+						       gboolean           override_browse_mode);
 gboolean     _gtk_tree_view_find_node                 (GtkTreeView       *tree_view,
 						       GtkTreePath       *path,
 						       GtkRBTree        **tree,
@@ -295,6 +296,8 @@ void _gtk_tree_view_column_realize_button   (GtkTreeViewColumn *column);
 void _gtk_tree_view_column_unrealize_button (GtkTreeViewColumn *column);
 void _gtk_tree_view_column_set_tree_view    (GtkTreeViewColumn *column,
 					     GtkTreeView       *tree_view);
+void _gtk_tree_view_column_unset_model      (GtkTreeViewColumn *column,
+					     GtkTreeModel      *old_model);
 void _gtk_tree_view_column_unset_tree_view  (GtkTreeViewColumn *column);
 void _gtk_tree_view_column_set_width        (GtkTreeViewColumn *column,
 					     gint               width);
@@ -310,7 +313,13 @@ gboolean _gtk_tree_view_column_cell_event   (GtkTreeViewColumn  *tree_column,
 void _gtk_tree_view_column_start_editing (GtkTreeViewColumn *tree_column,
 					  GtkCellEditable   *editable_widget);
 void _gtk_tree_view_column_stop_editing  (GtkTreeViewColumn *tree_column);
-void _gtk_tree_view_install_mark_rows_col_dirty (GtkTreeView *tree_view);					 
+void _gtk_tree_view_install_mark_rows_col_dirty (GtkTreeView *tree_view);
+void             _gtk_tree_view_column_autosize          (GtkTreeView       *tree_view,
+							  GtkTreeViewColumn *column);
+
+gboolean         _gtk_tree_view_column_has_editable_cell (GtkTreeViewColumn *column);
+GtkCellRenderer *_gtk_tree_view_column_get_editable_cell (GtkTreeViewColumn *column);
+gint             _gtk_tree_view_column_count_special_cells (GtkTreeViewColumn *column);
 
 GtkTreeSelection* _gtk_tree_selection_new                (void);
 GtkTreeSelection* _gtk_tree_selection_new_with_tree_view (GtkTreeView      *tree_view);
@@ -333,6 +342,10 @@ void		  _gtk_tree_view_column_cell_draw_focus  (GtkTreeViewColumn *tree_column,
 							  guint              flags);
 void		  _gtk_tree_view_column_cell_set_dirty	 (GtkTreeViewColumn *tree_column,
 							  gboolean           install_handler);
+void              _gtk_tree_view_column_get_neighbor_sizes (GtkTreeViewColumn *column,
+							    GtkCellRenderer   *cell,
+							    gint              *left,
+							    gint              *right);
 
 
 #ifdef __cplusplus
